"""
    CGI<>WSGI interface
    
    @copyright: 2009 by Scott Armitage <mail@scott.armitage.name>
    @license: GNU GPLv2, see COPYING for details

"""

# Standard Python modules -----------------------------------------------------
import os, sys

# Module function -------------------------------------------------------------
def run_wsgi(application):
    """
    Build a WSGI application environment from the existing CGI environment and
    run the specified application.
    
    Modified from <http://www.python.org/dev/peps/pep-0333/> which has been
    placed in the public domain.
    
    Parameters
    ----------
    application : callable
        The application must accept two ordered parameters: a dictionary object
        containing the WSGI environ and a callable for setting the response
        status and headers. The application must return an iterator yielding
        zero or more strings as the response content.
    
    Returns
    -------
    None
    
    """
    environ = dict(os.environ.items())
    environ['wsgi.input'] = sys.stdin
    environ['wsgi.errors'] = sys.stderr
    environ['wsgi.version'] = (1,0)
    environ['wsgi.multithread'] = False
    environ['wsgi.multiprocess'] = True
    environ['wsgi.run_once'] = True
    
    if environ.get('HTTPS','off') in ('on','1'):
        environ['wsgi.url_scheme'] = 'https'
    else:
        environ['wsgi.url_scheme'] = 'http'
    
    headers_set = []
    headers_sent = []
    
    def write(data):
        """Write the data to the output stream and flush."""
        if not headers_set: 
            # start_response() has not yet been called
            raise AssertionError("write() before start_response()")
        elif not headers_sent:
            # Before the first output, send the stored headers
            status, response_headers = headers_sent[:] = headers_set
            sys.stdout.write('Status: %s\r\n' % status)
            for header in response_headers:
                sys.stdout.write('%s: %s\r\n' % header)
            sys.stdout.write('\r\n')
        sys.stdout.write(data)
        sys.stdout.flush()

    def start_response(status, response_headers, exc_info=None):
        """Set the response headers to be sent on first call to write()."""
        if exc_info:
            try:
                if headers_sent:
                    # Re-raise original exception if headers sent
                    raise exc_info[0], exc_info[1], exc_info[2]
            finally:
                exc_info = None     # avoid dangling circular ref
        elif headers_set:
            raise AssertionError("Headers already set!")
        headers_set[:] = [status,response_headers]
        return write
    
    result = application(environ, start_response)
    try:
        for data in result:
            if data:    # don't send headers until body appears
                write(data)
        if not headers_sent:
            write('')   # send headers now if body was empty
    finally:
        if hasattr(result,'close'):
            result.close()
